@@ -1,6 +1,6 @@
-/* Totem Mozilla plugin
+/* Totem Mozilla Embedded Viewer
  *
- * Copyright (C) <2004-2005> Bastien Nocera <hadess@hadess.net>
+ * Copyright (C) <2004-2006> Bastien Nocera <hadess@hadess.net>
  * Copyright (C) <2002> David A. Schleef <ds@schleef.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -19,6 +19,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+//FIXME damn build system!
+#include "totem-interface.c"
+
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <stdlib.h>
@@ -27,6 +30,7 @@
 #include <gdk/gdkx.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
+#include <totem-pl-parser.h>
 
 #include <dbus/dbus-glib.h>
 #include <sys/types.h>
@@ -39,13 +43,15 @@
 #include "totem-interface.h"
 #include "totem-mozilla-options.h"
 #include "bacon-volume.h"
-//FIXME damn build system!
-#include "totem-interface.c"
 #include "video-utils.h"
 
 GtkWidget *totem_volume_create (void);
 
 #define OPTION_IS(x) (strcmp(argv[i], x) == 0)
+#define IS_FD_STREAM (strcmp(emb->filename, "fd://0") == 0)
+
+#define VOLUME_DOWN_OFFSET -8
+#define VOLUME_UP_OFFSET 8
 
 /* For newer D-Bus version */
 #ifndef DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT
@@ -59,7 +65,12 @@ typedef enum {
 } TotemStates;
 
 #define TOTEM_TYPE_EMBEDDED (totem_embedded_get_type ())
-typedef GObjectClass TotemEmbeddedClass;
+typedef struct {
+	GObjectClass parent_class;
+
+	void (*stop_sending_data) (GObject *emb);
+} TotemEmbeddedClass;
+
 typedef struct _TotemEmbedded {
 	GObject parent;
 
@@ -71,11 +82,15 @@ typedef struct _TotemEmbedded {
 	const char *mimetype;
 	char *filename, *href, *target;
 	BaconVideoWidget *bvw;
-	gboolean controller_hidden;
-	gboolean hidden;
 	TotemStates state;
 	GdkCursor *cursor;
 
+	/* Playlist */
+	gboolean is_playlist;
+	GList *playlist, *current;
+	GMainLoop *loop;
+	int num_items;
+
 	/* Open menu item */
 	GnomeVFSMimeApplication *app;
 	GtkWidget *menu_item;
@@ -83,15 +98,18 @@ typedef struct _TotemEmbedded {
 	/* Seek bits */
 	GtkAdjustment *seekadj;
 	GtkWidget *seek;
-	gboolean seeking;
 
-	/* XEmbed */
-	gboolean embedded_done;
+	guint embedded_done : 1;
+	guint controller_hidden : 1;
+	guint hidden : 1;
+	guint repeat : 1;
+	guint seeking : 1;
+	guint noautostart : 1;
 } TotemEmbedded;
+
 GType totem_embedded_get_type (void);
 
 G_DEFINE_TYPE (TotemEmbedded, totem_embedded, G_TYPE_OBJECT);
-static void totem_embedded_class_init (TotemEmbeddedClass *klass) { }
 static void totem_embedded_init (TotemEmbedded *emb) { }
 
 gboolean totem_embedded_play (TotemEmbedded *emb, GError **err);
@@ -104,6 +122,28 @@ static void on_open1_activate (GtkButton *button, TotemEmbedded *emb);
 
 #include "totem-mozilla-interface.h"
 
+enum {
+	STOP_SENDING_DATA,
+	LAST_SIGNAL
+};
+static int totem_emb_table_signals[LAST_SIGNAL] = { 0 };
+
+static void totem_embedded_class_init (TotemEmbeddedClass *klass)
+{
+	GObjectClass *object_class;
+
+	object_class = (GObjectClass *) klass;
+
+	totem_emb_table_signals[STOP_SENDING_DATA] =
+		g_signal_new ("stop-sending-data",
+				G_TYPE_FROM_CLASS (object_class),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (TotemEmbeddedClass, stop_sending_data),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
+}
+
 static void
 totem_embedded_exit (TotemEmbedded *emb)
 {
@@ -119,6 +159,17 @@ totem_embedded_error_and_exit (char *title, char *reason, TotemEmbedded *emb)
 	totem_embedded_exit (emb);
 }
 
+static void
+totem_embedded_emit_stop_sending_data (TotemEmbedded *emb)
+{
+	g_return_if_fail (emb->filename != NULL);
+	if (IS_FD_STREAM) {
+		g_signal_emit (G_OBJECT (emb),
+				totem_emb_table_signals[STOP_SENDING_DATA],
+				0, NULL);
+	}
+}
+
 static void
 totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 {
@@ -149,7 +200,7 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 
 	image = glade_xml_get_widget (emb->xml, "emb_pp_button_image");
 	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_MENU);
-	if (emb->hidden == FALSE)
+	if (emb->hidden == FALSE && cursor)
 		gdk_window_set_cursor (GTK_WIDGET (emb->bvw)->window, cursor);
 
 	emb->state = state;
@@ -177,6 +228,8 @@ totem_embedded_open (TotemEmbedded *emb)
 	{
 		char *msg, *disp;
 
+		totem_embedded_emit_stop_sending_data (emb);
+
 		totem_embedded_set_state (emb, STATE_STOPPED);
 
 		//FIXME if emb->filename is fd://0 or stdin:///
@@ -192,15 +245,15 @@ totem_embedded_open (TotemEmbedded *emb)
 		}
 
 		g_free (msg);
-
 		g_error_free (err);
+
 		totem_embedded_set_pp_state (emb, FALSE);
 	} else {
 		totem_embedded_set_state (emb, STATE_PAUSED);
 		totem_embedded_set_pp_state (emb, TRUE);
 	}
 
-	if (retval == FALSE || strcmp ("fd://0", emb->filename) == 0) {
+	if (retval == FALSE || IS_FD_STREAM) {
 		totem_embedded_set_menu (emb, FALSE);
 	} else {
 		totem_embedded_set_menu (emb, TRUE);
@@ -327,12 +380,12 @@ on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 
 	emb->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
 				   "name", _("Totem Mozilla Plugin"),
-				   "version", PACKAGE_VERSION,
+				   "version", VERSION,
 				   "copyright", _("Copyright \xc2\xa9 2002-2006 Bastien Nocera"),
 				   "comments", description,
 				   "authors", authors,
 				   "translator-credits", _("translator-credits"),
-				   "logo-icon-name", "media-player-48",
+				   "logo-icon-name", "totem",
 				   NULL);
 
 	g_free (backend_version);
@@ -461,11 +514,45 @@ on_eos_event (GtkWidget *bvw, TotemEmbedded *emb)
 {
 	totem_embedded_set_state (emb, STATE_STOPPED);
 	gtk_adjustment_set_value (emb->seekadj, 0);
-	if (strcmp (emb->filename, "fd://0") == 0) {
+
+	/* No playlist if we have fd://0, right? */
+	if (IS_FD_STREAM) {
 		totem_embedded_set_pp_state (emb, FALSE);
-	} else if (g_str_has_prefix (emb->filename, "file://") != FALSE) {
+	} else if (emb->num_items == 1) {
+		if (g_str_has_prefix (emb->filename, "file://") != FALSE) {
+			if (bacon_video_widget_is_seekable (emb->bvw) != FALSE) {
+				bacon_video_widget_pause (emb->bvw);
+				bacon_video_widget_seek (emb->bvw, 0.0, NULL);
+			} else {
+				bacon_video_widget_close (emb->bvw);
+				totem_embedded_open (emb);
+			}
+		} else {
+			bacon_video_widget_close (emb->bvw);
+			totem_embedded_open (emb);
+		}
+		if (emb->repeat != FALSE && emb->noautostart == FALSE)
+			totem_embedded_play (emb, NULL);
+	} else {
+		/* Multiple items on the playlist */
+		gboolean eop = FALSE, res;
+
+		if (emb->current->next == NULL) {
+			emb->current = emb->playlist;
+			eop = TRUE;
+		} else {
+			emb->current = emb->current->next;
+		}
+
+		g_free (emb->filename);
+		emb->filename = g_strdup (emb->current->data);
 		bacon_video_widget_close (emb->bvw);
-		totem_embedded_open (emb);
+		res = totem_embedded_open (emb);
+		if (res != FALSE &&
+				((eop != FALSE && emb->repeat != FALSE)
+				 || (eop == FALSE))) {
+			totem_embedded_play (emb, NULL);
+		}
 	}
 }
 
@@ -476,6 +563,31 @@ cb_vol (GtkWidget *val, TotemEmbedded *emb)
 		bacon_volume_button_get_value (BACON_VOLUME_BUTTON (val)));
 }
 
+static gboolean
+on_volume_scroll_event (GtkWidget *win, GdkEventScroll *event, TotemEmbedded *emb)
+{
+	GtkWidget *vbut;
+	int vol, offset;
+
+	switch (event->direction) {
+	case GDK_SCROLL_UP:
+		offset = VOLUME_UP_OFFSET;
+		break;
+	case GDK_SCROLL_DOWN:
+		offset = VOLUME_DOWN_OFFSET;
+		break;
+	default:
+		return FALSE;
+	}
+
+	vbut = glade_xml_get_widget (emb->xml, "volume_button");
+	vol = bacon_volume_button_get_value (BACON_VOLUME_BUTTON (vbut));
+	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (vbut),
+			vol + offset);
+
+	return FALSE;
+}
+
 static void
 on_tick (GtkWidget *bvw,
 		gint64 current_time,
@@ -581,6 +693,9 @@ totem_embedded_add_children (TotemEmbedded *emb)
 	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (vbut), volume);
 	g_signal_connect (G_OBJECT (vbut), "value-changed",
 			  G_CALLBACK (cb_vol), emb);
+	gtk_widget_add_events (vbut, GDK_SCROLL_MASK);
+	g_signal_connect (G_OBJECT (vbut), "scroll_event",
+			G_CALLBACK (on_volume_scroll_event), emb);
 
 	gtk_widget_realize (emb->window);
 	gtk_widget_set_size_request (emb->window, emb->width, emb->height);
@@ -616,6 +731,70 @@ totem_embedded_create_cursor (TotemEmbedded *emb)
 	gdk_pixbuf_unref (icon);
 }
 
+static void
+entry_added (TotemPlParser *parser, const char *uri, const char *title,
+		                const char *genre, gpointer data)
+{
+	TotemEmbedded *emb = (TotemEmbedded *) data;
+
+	g_print ("added URI '%s' with title '%s' genre '%s'\n", uri,
+			title ? title : "empty", genre);
+
+	//FIXME need new struct to hold that
+	emb->playlist = g_list_prepend (emb->playlist, g_strdup (uri));
+	emb->num_items++;
+}
+
+static gboolean
+totem_embedded_push_parser (gpointer data)
+{
+	TotemPlParser *parser = totem_pl_parser_new ();
+	TotemEmbedded *emb = (TotemEmbedded *) data;
+	TotemPlParserResult res;
+
+	parser = totem_pl_parser_new ();
+	g_signal_connect (G_OBJECT (parser), "entry", G_CALLBACK (entry_added), emb);
+	res = totem_pl_parser_parse (parser, emb->filename, FALSE);
+	g_object_unref (parser);
+
+	if (res != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+		//FIXME show a proper error message
+		switch (res) {
+		case TOTEM_PL_PARSER_RESULT_UNHANDLED:
+			g_print ("url '%s' unhandled\n", emb->filename);
+			break;
+		case TOTEM_PL_PARSER_RESULT_ERROR:
+			g_print ("error handling url '%s'\n", emb->filename);
+			break;
+		case TOTEM_PL_PARSER_RESULT_IGNORED:
+			g_print ("ignored url '%s'\n", emb->filename);
+			break;
+		default:
+			g_assert_not_reached ();
+			;;
+		}
+	}
+
+	/* Check if we have anything in the playlist now */
+	if (emb->playlist == NULL) {
+		g_message ("NO PLAYLIST");
+		totem_embedded_error_and_exit ("Can't parse that",
+				"no files",
+				emb);
+		//FIXME error out
+		return FALSE;
+	}
+
+	emb->playlist = g_list_reverse (emb->playlist);
+	g_free (emb->filename);
+	emb->filename = g_strdup (emb->playlist->data);
+	emb->current = emb->playlist;
+
+	g_main_loop_quit (emb->loop);
+
+	return FALSE;
+}
+
 static void embedded (GtkPlug *plug, TotemEmbedded *emb)
 {
 	emb->embedded_done = TRUE;
@@ -733,6 +912,12 @@ int main (int argc, char **argv)
 			}
 		} else if (OPTION_IS (TOTEM_OPTION_HIDDEN)) {
 			emb->hidden = TRUE;
+		} else if (OPTION_IS (TOTEM_OPTION_PLAYLIST)) {
+			emb->is_playlist = TRUE;
+		} else if (OPTION_IS (TOTEM_OPTION_REPEAT)) {
+			emb->repeat = TRUE;
+		} else if (OPTION_IS (TOTEM_OPTION_NOAUTOSTART)) {
+			emb->noautostart = TRUE;
 		} else if (i + 1 == argc) {
 			emb->filename = g_strdup (argv[i]);
 		}
@@ -771,8 +956,22 @@ int main (int argc, char **argv)
 			gtk_main_iteration ();
 	}
 
-	if (totem_embedded_open (emb) != FALSE)
+	/* Do we have a playlist we need to setup ourselves? */
+	if (emb->is_playlist != FALSE) {
+		g_idle_add (totem_embedded_push_parser, emb);
+		emb->loop = g_main_loop_new (NULL, FALSE);
+		g_main_loop_run (emb->loop);
+		g_main_loop_unref (emb->loop);
+	} else {
+		emb->playlist = g_list_prepend (emb->playlist,
+				g_strdup (emb->filename));
+		emb->num_items++;
+	}
+
+	if (totem_embedded_open (emb) != FALSE
+			&& emb->noautostart == FALSE) {
 		totem_embedded_play (emb, NULL);
+	}
 
 	gtk_main ();
 
